@@ -24,6 +24,7 @@ class PersistedRmq(object):
         self.queue_key = f'{self.channel}-{self.client_id}-queue'
         self.subscribe_chn = f'chn:{self.channel}'
         self.persisted_timeout = 3600 * 24 * 3
+        self.running = True
 
     async def __aenter__(self):
         await self.init_mq()
@@ -39,6 +40,10 @@ class PersistedRmq(object):
         self.conn.close()
         await self.conn.wait_closed()
 
+    async def stop(self):
+        self.running = False
+        await self.conn.unsubscribe(self.subscribe_chn)
+
     async def connect(self):
         self.conn = await aioredis.create_redis_pool(self.__class__.redis_uri)
 
@@ -112,8 +117,8 @@ class PersistedRmq(object):
         await self.set('client-map', self.lock_key, self.timestamp)
         logging.error(f'进入订阅:{self.subscribe_chn}')
         chs = await self.conn.subscribe(self.subscribe_chn)
-        while True:
-            while await chs[0].wait_message():
+        while self.running:
+            if await chs[0].wait_message():
                 msg = await chs[0].get()
                 try:
                     await self.on_message(msg)
@@ -130,6 +135,9 @@ class PersistedRmq(object):
                         logging.error(f'{self.queue_key} 下发通道关闭，向错误队列push')
                     await self.close()
                     return
+            else:
+                break
+        await self.close()
 
     async def publish(self, message, another_chn=None):
         keys = []
@@ -153,6 +161,7 @@ class PersistedRmq(object):
                 logging.error(f'pub queue:{queue_key}')
                 if queue_key:
                    keys.append(queue_key)
+
         for key in keys:
             await self.conn.rpush(queue_key, message)
             logging.error(f'往失败队列节点{key}push')
